package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectHistory;
import cn.com.ty.lift.business.project.dao.entity.ProjectUser;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectHistoryRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectRequest;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectHistoryMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectUserMapper;
import cn.com.ty.lift.common.base.ExportRequest;
import cn.com.ty.lift.common.constants.ApiConstants;
import cn.com.ty.lift.common.constants.RedisConstants;
import cn.com.ty.lift.common.export.ExportUtils;
import cn.com.ty.lift.common.model.AreaCode;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author bieao
 * @date 2019/11/28
 * @description 项目管理业务层
 */
@Service
public class ProjectService {

    @Resource
    private ProjectMapper projectMapper;

    @Resource
    private ProjectUserMapper projectUserMapper;

    @Resource
    private ProjectHistoryMapper projectHistoryMapper;

    @Resource
    private RedisTemplate redisTemplate;


    private Map<String, String> paramMap = new HashMap<String, String>() {{
        put("projectCode", "项目编号");
        put("projectName", "项目名称");
        put("province", "省份");
        put("city", "市");
        put("district", "区");
        put("num", "台量");
        put("actualNum", "系统台量");
        put("address", "项目地址");
        put("startDate", "开始时间");
        put("endDate", "结束时间");
    }};

    /**
     * @param request 项目列表查询条件
     * @return RestResponse 结果集
     * @description 条件查询项目管理列表
     * @date 2019/11/27 2:10 PM
     */
    public RestResponse list(ProjectRequest request) {
        IPage<ProjectResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        List<ProjectResponse> projectList = projectMapper.findByCondition(page, request);
        if (projectList.isEmpty()) {
            page.setRecords(new ArrayList<>());
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, MessageUtils.get("msg.data.empty"));
        }
        page.setRecords(projectList);
        return RestResponse.ok(page, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 公司项目查询条件
     * @return RestResponse 公司项目分页列表结果
     * @description 查询公司项目列表
     * @date 2019/12/9 10:32 AM
     */
    public RestResponse companyProjectList(ProjectRequest request) {
        IPage<ProjectResponse> page = new Page<>(request.getPageNum(), request.getPageSize());
        List<ProjectResponse> projectList = projectMapper.findCompanyListByCondition(page, request);
        if (projectList.isEmpty()) {
            page.setRecords(new ArrayList<>());
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, MessageUtils.get("msg.data.empty"));
        }
        page.setRecords(projectList);
        return RestResponse.ok(page, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param request 项目操作记录查询条件
     * @return RestResponse 结果集
     * @description 查询项目操作记录列表
     * @date 2019/11/27 2:10 PM
     */
    public RestResponse list(ProjectHistoryRequest request) {
        IPage<ProjectHistory> page = new Page<>(request.getPageNum(), request.getPageSize());
        List<ProjectHistory> historyList = projectHistoryMapper.findByCondition(page, request);
        if (historyList.isEmpty()) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, MessageUtils.get("msg.data.empty"));
        }
        page.setRecords(historyList);
        return RestResponse.ok(page, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param id 项目id
     * @return RestResponse 项目详情
     * @description 查询项目详情
     * @date 2019/12/6 10:51 AM
     */
    public RestResponse detail(Long id) {
        Project project = projectMapper.selectById(id);
        if (project == null) {
            return RestResponse.ok(null, ApiConstants.RESULT_NO_DATA, MessageUtils.get("msg.data.empty"));
        }
        return RestResponse.ok(project, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.detail.success"));
    }

    /**
     * @return RestResponse 省、市、区树形结构列表
     * @description 查询省、市、区树形结构列表
     * @date 2019/12/4 10:23 AM
     */
    public RestResponse areaList() {
        List<AreaCode> areaCodeList = (List<AreaCode>) redisTemplate.opsForValue().get(RedisConstants.RK_AREA_LIST);
        return RestResponse.ok(areaCodeList, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.query.success"));
    }

    /**
     * @param project 新增项目数据项
     * @return RestResponse 状态码和返回消息
     * @description 新增项目
     * @date 2019/11/27 2:22 PM
     */
    public RestResponse add(Project project) {
        int result = projectMapper.insert(project);
        if (result > 0) {
            return RestResponse.ok(result, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.add.success"));
        }
        return RestResponse.error(ApiConstants.RESULT_ERROR, MessageUtils.get("msg.add.fail"));
    }

    /**
     * @param projectUser 新增维保工数据项
     * @return RestResponse 状态码和返回消息
     * @description 新增维保工
     * @date 2019/12/4 4:41 PM
     */
    public RestResponse add(ProjectUser projectUser){
        int result = projectUserMapper.insert(projectUser);
        if (result > 0) {
            return RestResponse.ok(result, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.add.success"));
        }
        return RestResponse.error(ApiConstants.RESULT_ERROR, MessageUtils.get("msg.add.fail"));
    }

    /**
     * @param project 修改项目数据项
     * @return RestResponse 状态码和返回消息
     * @description 修改项目
     * @date 2019/11/27 2:22 PM
     */
    public RestResponse modify(Project project) {
        int result = projectMapper.updateById(project);
        if (result > 0) {
            return RestResponse.ok(result, ApiConstants.RESULT_SUCCESS, MessageUtils.get("msg.modify.success"));
        }
        return RestResponse.error(ApiConstants.RESULT_ERROR, MessageUtils.get("msg.modify.fail"));
    }

    /**
     * @param request 导出项目数据列表
     * @return RestResponse 状态码和返回消息
     * @description 导出项目
     * @date 2019/11/27 2:22 PM
     */
    public void export(ExportRequest request) {
        List<Project> projectList = projectMapper.selectBatchIds(request.getIds());
        ExportUtils utils = new ExportUtils();
        utils.export(projectList, paramMap, request.getExportPath());
    }
}
